# Copyright (c) 2020, Oracle and/or its affiliates. 
# Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upD
"""
autonmous linux related utility functions.

"""
from backend.command_util import execute_command, command_output_lines
from backend.common import (AUTONOMOUS_OCI_CONFIG_FILE, CMD_AL_CONFIG)
from backend.log_util import get_logger, set_log_config
from backend.utility import get_full_lv_path, get_full_mount_path, round_to_multiplier
from backend.fs_util import get_share_stat, mount_lv, umount_lv
import os

LOG = get_logger('lsa.autonomous')


def get_notification_topic():
    '''
    Retrieve the current OCI topic OCID used by al-config to send notification
    Raise exception on failure. Caller should handle the exception
    output: a OCID string
    '''
    lines = []
    fobj = None

    try:
        fobj = open(AUTONOMOUS_OCI_CONFIG_FILE)
    except Exception:
        return None
            
    lines = fobj.readlines()
    fobj.close()
    for line in lines:
        line = line.strip()
        if not line:
            continue
        # expected format if line exists topic="ocid1.onstopic.oc1.iad.blah"
        if 'topic=' in line:
            parts = line.split('"')
            if len(parts) >= 2:
                return parts[1]
    return None

def set_notification_topic(topic_ocid):
    '''
    Calls al-config -T topic_ocid
    input: topic_ocid, a string representing an OCI notification topic OCID 
       like: ocid1.onstopic.oc1.iad.blah
           An empty string removes the notification OCID
    raises an exception on failure
    '''
    if topic_ocid:
        cmd = [CMD_AL_CONFIG, '-T', topic_ocid]
        out, err, code = execute_command(cmd)
        if code != 0 or err != b'':
            error = 'set_notification_topic failure %s'  % err.decode()
            LOG.warning(error)
            raise Exception(error)
        return out.decode()
    else: # remove topic from file
        if os.path.exists(AUTONOMOUS_OCI_CONFIG_FILE):
            input_file = None
            try:
                input_file = open(AUTONOMOUS_OCI_CONFIG_FILE,"r")
            except Exception:
                return None
            if input_file:
                lines = input_file.readlines()
                input_file.close()
            else:
                return None
            output_file = None
            try:
                output_file = open(AUTONOMOUS_OCI_CONFIG_FILE,"w")
            except Exception:
                return None
            if output_file:
                for line in lines:
                    if 'topic=' not in line:
                        output_file.write(line)
                output_file.close()

    return None


def get_update_time_info():
    '''
    Returns the current time window for al-config update and the current update time
    output: [start window time 24h, end window time 24h, update time 24h]
    '''
    cmd = [CMD_AL_CONFIG, '-s']
    out, err, code = execute_command(cmd)
    if code != 0 and err != b'':
        error = 'get_update_time_info failure %s'  % err.decode()
        LOG.warning(error)
        raise Exception(error)
    # extracting info from this format b'Current daily auto update time window(24-hour): 5-8\nCurrent daily auto update time(24-hour): 05:18\n'
    lines = out.decode().split('\n')
    update_window = lines[0].split(': ')[1].split('-')
    update_time = lines[1].split(': ')[1]
    return [update_window[0], update_window[1], update_time]


def set_update_time_info(start_time, end_time):
    '''
    Set a new update time window
    raises an exception on failure
    '''
    cmd = [CMD_AL_CONFIG, '-w', '%s-%s' % (start_time, end_time)]
    out, err, code = execute_command(cmd)
    if code != 0 or err != b'':
        error = 'get_update_time_info failure %s' % err.decode()
        LOG.warning(error)
        raise Exception(error)


if __name__ == '__main__':
    set_log_config()
    print (get_notification_topic())
