/**
 * Copyright (c) 2020, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'viewModels/common_vm_def',
    'models/administration',
    'viewModels/confirmation-dialog',
    'models/platform',
    'ojs/ojinputtext',
    'ojs/ojlabel',
    'ojs/ojdialog',
    'ojs/ojswitch',
    'ojs/ojvalidation-base'],
    function (oj, ko, $, CommonVMDefinitons, adminSVC, confirmation, Platform) {
        function NotificationOCIDViewModel() {
            var self = this;
            this.notificationOCID = ko.observable('');

            this.open = function () {
                new adminSVC().getNotificationOCID().then(function (ocid) {
                    reset_all_info();
                    self.notificationOCID(ocid);
                    $(self.dialogSelector).ojDialog('open');
                }).catch(function (why) {
                    oj.Logger.error('Failed to retrieve notification OCID');
                    confirmation.showDialog(
                        {
                            'notifyOnly': true,
                            'title': oj.Translations.getTranslatedString('Error'),
                            'message': oj.Translations.getTranslatedString('Failed to retrieve notification OCID: ' + why)
                        });
                }
                );
            };

            /*
                reset all the parameters to empty value
             */
            function reset_all_info() {
                self.notificationOCID('');
            };

            // Validator for fully qualified domain name (FQDN)
            this.notificationOCIDValidator = {
                getHint: function () {
                    return self.fqdnValidationHintMessage;
                },
                errorRaised: ko.observable(false),
                validate: function (value) {
                    this.errorRaised(false);
                    if (value.includes('.onstopic.') || value === "") {
                        return true;
                    }
                    this.errorRaised(true);
                    throw new oj.ValidatorError(self.ocidValidationHintMessage);
                }
            };

            // Check if we should disable the update button based on the user input
            this.shouldDisableCreateButton = ko.pureComputed(function () {
                var value = self.notificationOCID();
                if (value == null)
                    return false;
                if ((value.includes('.onstopic.') || value === ""))
                    return false;
                return true;
            });

            this.handleCancelClick = function () {
                reset_all_info();
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleCloseDrawerClick = function () {
                reset_all_info();
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleUpdateClick = function (data, event) {
                confirmation.showDialog({
                    'title': oj.Translations.getTranslatedString('Topic OCID update'),
                    'message': oj.Translations.getTranslatedString('Are you sure you want to update Notification topic OCID ?'),
                    'onOK': function () {
                        new adminSVC().setNotificationOCID(self.notificationOCID()).then(function (payload) {
                            oj.Logger.info('Notification Topic OCID was updated successfully');
                            confirmation.showDialog({
                                'notifyOnly': true,
                                'title': oj.Translations.getTranslatedString('Successful Modification'),
                                'message': oj.Translations.getTranslatedString('Notification Topic OCID was updated successfully')
                            });

                        }).catch(function (why) {
                            oj.Logger.error('Updating Notification Topic OCID failed: ' + why);
                            confirmation.showDialog({
                                'notifyOnly': true,
                                'title': oj.Translations.getTranslatedString('Notification Topic OCID modification failure'),
                                'message': why
                            });
                        }
                        );
                        $(self.dialogSelector).ojDialog("close");
                    }
                });
            };
        }

        $.extend(NotificationOCIDViewModel.prototype, CommonVMDefinitons.prototype.definitions);

        NotificationOCIDViewModel.prototype.dialogSelector = '#notificationOCIDDialog';
        NotificationOCIDViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Autonomous Topic notification settings');
        NotificationOCIDViewModel.prototype.notificationOCIDOptionLabel = oj.Translations.getTranslatedString('Topic Notification OCID');
        NotificationOCIDViewModel.prototype.notificationOCIDHelp = oj.Translations.getTranslatedString('Specify the OCI Topic notification OCID that the appliance will use to send autonomous update information. An empty field unsets the current notification OCID. Click for more information.');
        NotificationOCIDViewModel.prototype.notificationOCIDValidationHintMessage = oj.Translations.getTranslatedString('topic OCID');
        NotificationOCIDViewModel.prototype.notificationOCIDSource = 'https://docs.cloud.oracle.com/en-us/iaas/Content/Notification/Tasks/managingtopicsandsubscriptions.htm';
        NotificationOCIDViewModel.prototype.notificationOCIDUpdateLabel = oj.Translations.getTranslatedString('Set');
        NotificationOCIDViewModel.prototype.ocidValidationHintMessage = oj.Translations.getTranslatedString('OCID format should be similar to ocid1.onstopic.oc.....');

        return new NotificationOCIDViewModel();
    }
);
